/****************************************************************************/
/*                         SQL Server Internals v3                          */
/*                           Training Materials                             */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                       Objects Creation Script                            */
/****************************************************************************/

set nocount on
go

use SQLServerInternals
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'LargeRow'
)
	drop table dbo.LargeRow
go

create table dbo.LargeRow
(
	ID int not null,
	IntField int not null, 
	CharField char(2000) not null,
)
go

create unique clustered index IDX_LargeRow_ID
on dbo.LargeRow(ID)
with (/*drop_existing = on, */data_compression = none)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'dbo' and t.name = 'SmallRow'
)
	drop table dbo.SmallRow
go

create table dbo.SmallRow
(
	ID int not null,
	IntField int not null, 
	CharField varchar(2000) not null,
)
go


create unique clustered index IDX_SmallRow_ID
on dbo.SmallRow(ID)
with (/*drop_existing = on, */data_compression = none)
go

;WITH N1(C) AS (SELECT 0 UNION ALL SELECT 0) -- 2 rows
,N2(C) AS (SELECT 0 FROM N1 AS T1 CROSS JOIN N1 AS T2) -- 4 rows
,N3(C) AS (SELECT 0 FROM N2 AS T1 CROSS JOIN N2 AS T2) -- 16 rows
,N4(C) AS (SELECT 0 FROM N3 AS T1 CROSS JOIN N3 AS T2) -- 256 rows
,N5(C) AS (SELECT 0 FROM N4 AS T1 CROSS JOIN N4 AS T2) -- 65,536 rows
,IDs(ID) AS (SELECT ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) FROM N5)
insert into dbo.LargeRow(ID, IntField,CharField) 
	select ID, ID, 'Placeholer'
	from IDs
go
	
;WITH N1(C) AS (SELECT 0 UNION ALL SELECT 0) -- 2 rows
,N2(C) AS (SELECT 0 FROM N1 AS T1 CROSS JOIN N1 AS T2) -- 4 rows
,N3(C) AS (SELECT 0 FROM N2 AS T1 CROSS JOIN N2 AS T2) -- 16 rows
,N4(C) AS (SELECT 0 FROM N3 AS T1 CROSS JOIN N3 AS T2) -- 256 rows
,N5(C) AS (SELECT 0 FROM N4 AS T1 CROSS JOIN N4 AS T2) -- 65,536 rows
,IDs(ID) AS (SELECT ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) FROM N5)
insert into dbo.SmallRow(ID, IntField,CharField) 
	select ID, ID, 'Placeholer'
	from IDs
go
	

-- Delivery schema
if not exists (select * from sys.schemas where name = 'Delivery')
	exec sp_executesql N'CREATE SCHEMA [Delivery] AUTHORIZATION [dbo]'
GO

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'Customers'
)
	drop table Delivery.Customers
go

create table Delivery.Customers
(
	CustomerID int not null identity(1,1),
	Name varchar(100) not null,
	Phone varchar(20) not null,
	ContactName varchar(100) not null,
	BillingAddress varchar(100) not null,
	BillingCity varchar(40) not null,
	BillingState char(2) not null,
	BillingZip char(5) not null,
	DefaultRatePlan int null,
	DefaultService int null,
	RegionId int not null,
		
	constraint PK_Customers
	primary key clustered(CustomerID)
)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'Addresses'
)
	drop table Delivery.Addresses
go

create table Delivery.Addresses
(
	AddressId int not null identity(1,1),
	CustomerId int not null,
	Address varchar(100) not null,
	City varchar(40) not null,
	State char(2) not null,
	Zip char(5) not null,
	Direction varchar(1024) null,	

	constraint PK_Addresses
	primary key clustered(AddressId)
)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'Services'
)
	drop table Delivery.Services
go

create table Delivery.Services
(
	ServiceId int not null,
	Name varchar(40) not null,
	
	constraint PK_Services
	primary key clustered(ServiceId)
)
go


if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'RatePlans'
)
	drop table Delivery.RatePlans
go

create table Delivery.RatePlans
(
	RatePlanId int not null,
	Name varchar(40) not null,
	
	constraint PK_RatePlans
	primary key clustered(RatePlanId)
)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'Rates'
)
	drop table Delivery.Rates
go

create table Delivery.Rates
(
	RatePlanId int not null,
	ServiceId int not null,
	Rate smallmoney,
	
	constraint PK_Rates
	primary key clustered(RatePlanId, ServiceId)
)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'Drivers'
)
	drop table Delivery.Drivers
go

create table Delivery.Drivers
(
	DriverId int not null identity(1,1),
	Name varchar(40) not null,

	constraint PK_Drivers
	primary key clustered(DriverId)
)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'OrderStatuses'
)
	drop table Delivery.OrderStatuses
go

create table Delivery.OrderStatuses
(
	OrderStatusId int not null,
	Name varchar(40) not null,
	
	constraint PK_OrderStatuses
	primary key clustered(OrderStatusId)
)
go

if exists(
	select * 
	from sys.tables t join sys.schemas s on t.schema_id = s.schema_id
	where s.name = 'Delivery' and t.name = 'Orders'
)
	drop table Delivery.Orders
go

create table Delivery.Orders
(
	OrderId int not null identity(1,1),
	OrderDate smalldatetime not null, 
	OrderNum varchar(20) not null,
	Reference varchar(64) null,
	CustomerId int not null,
	PickupAddressId int not null,
	DeliveryAddressId int not null,
	ServiceId int not null,
	RatePlanId int not null,
	OrderStatusId int not null,
	DriverId int null,
	Pieces smallint not null,
	Amount smallmoney not null,
	ModTime datetime2(0) not null
		constraint DEF_Orders_ModTime
		default getDate(),
	PlaceHolder char(100) not null
		constraint DEF_Orders_Placeholder
		default 'Placeholder',
		
	constraint PK_Orders
	primary key clustered(OrderId)
)
go

if exists(
	select * 
	from sys.procedures p join sys.schemas s on p.schema_id = s.schema_id
	where s.name = 'Delivery' and p.name = 'InsertAddress'
)
	drop proc Delivery.InsertAddress
go

create proc Delivery.InsertAddress
(
	@CustomerId int, 
	@AddressId int output
)
as
begin
	set nocount on
	set xact_abort on
	
	begin tran
		insert into Delivery.Addresses(CustomerId,Address,City,State,Zip,Direction) 
		values(@CustomerId, '456 Main Street', 'Tampa', 'FL', '33607', REPLICATE(' ',200));
		select @AddressId = @@identity
	commit
end
go

-- Random number generation
if OBJECT_ID(N'Delivery.Rand') is not null
	drop function Delivery.Rand
go

if exists(
	select * 
	from sys.views v join sys.schemas s on v.schema_id = s.schema_id
	where s.name = 'Delivery' and v.name = 'vRand'
)
	drop view Delivery.vRand
go


CREATE VIEW Delivery.vRand(RandNum)
AS
	SELECT RAND() as RandNum
go

create function Delivery.Rand(@MaxVal int)
returns int
as
begin
	return (select convert(int,RandNum) * @MaxVal + 1 from Delivery.vRand)
end
go


begin tran
	insert into Delivery.OrderStatuses(OrderStatusId, Name)
	values(1,'NEW'),(2,'DISPATCHED'),(3,'IN ROUTE'),(4,'COMPLETED')
		
	insert into Delivery.Services(ServiceId, Name)
	values(1,'BASE'),(2,'RUSH'),(3,'2-HOURS') 
		
	insert into Delivery.RatePlans(RatePlanId, Name)
	values(1,'REGULAR'),(2,'CORPORATE') 

	insert into Delivery.Rates(ServiceId, RatePlanId, Rate)
	values(1, 1, 15),(1, 2, 10),(2, 1, 25),(2, 2, 20),(3, 1, 35),(3, 2, 30)
		
	declare
		@MaxCustomers int = 1000
	
	;WITH N1(C) AS (SELECT 0 UNION ALL SELECT 0) -- 2 rows
	,N2(C) AS (SELECT 0 FROM N1 AS T1 CROSS JOIN N1 AS T2) -- 4 rows
	,N3(C) AS (SELECT 0 FROM N2 AS T1 CROSS JOIN N2 AS T2) -- 16 rows
	,N4(C) AS (SELECT 0 FROM N3 AS T1 CROSS JOIN N3 AS T2) -- 256 rows
	,N5(C) AS (SELECT 0 FROM N4 AS T1 CROSS JOIN N4 AS T2) -- 65,536 rows
	,IDs(ID) AS (SELECT ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) FROM N5)
	insert into Delivery.Customers(Name, Phone, ContactName, BillingAddress
		,BillingCity, BillingState, BillingZip, DefaultRatePlan, DefaultService, RegionId)
		select 
			'Customer # ' + convert(varchar(5), ID), '813-123-4567',
			'Contact for Customer # ' + convert(varchar(5), ID),
			'123 Main Street', 'Tampa', 'FL','33607',
			ID % 3, 1, ID % 50
		from IDs
		where ID <= @MaxCustomers

	declare
		@MaxAddresses int = 20
		
	;WITH N1(C) AS (SELECT 0 UNION ALL SELECT 0) -- 2 rows
		,N2(C) AS (SELECT 0 FROM N1 AS T1 CROSS JOIN N1 AS T2) -- 4 rows
		,N3(C) AS (SELECT 0 FROM N2 AS T1 CROSS JOIN N2 AS T2) -- 16 rows
		,N4(C) AS (SELECT 0 FROM N3 AS T1 CROSS JOIN N3 AS T2) -- 256 rows
		,N5(C) AS (SELECT 0 FROM N4 AS T1 CROSS JOIN N4 AS T2) -- 65,536 rows
		,IDs(ID) AS (SELECT ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) FROM N5)
	insert into Delivery.Addresses(CustomerId,Address,City,State,Zip,Direction)
		select convert(int, ID / @MaxAddresses) + 1, '456 Main Street', 'Tampa', 'FL', '33607', REPLICATE(' ',200)
		from IDs 
		where ID <= @MaxAddresses * @MaxCustomers
		order by ID

	declare
		@MaxDrivers int = 100
		
	;WITH N1(C) AS (SELECT 0 UNION ALL SELECT 0) -- 2 rows
		,N2(C) AS (SELECT 0 FROM N1 AS T1 CROSS JOIN N1 AS T2) -- 4 rows
		,N3(C) AS (SELECT 0 FROM N2 AS T1 CROSS JOIN N2 AS T2) -- 16 rows
		,N4(C) AS (SELECT 0 FROM N3 AS T1 CROSS JOIN N3 AS T2) -- 256 rows
		,IDs(ID) AS (SELECT ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) FROM N4)
	insert into Delivery.Drivers(Name)
		select 'Driver # ' + convert(varchar(5), ID)
		from IDs 
		where ID <= @MaxDrivers
	
	
	declare
		@MaxOrderId int = 65536 * 16

	;WITH N1(C) AS (SELECT 0 UNION ALL SELECT 0) -- 2 rows
	,N2(C) AS (SELECT 0 FROM N1 AS T1 CROSS JOIN N1 AS T2) -- 4 rows
	,N3(C) AS (SELECT 0 FROM N2 AS T1 CROSS JOIN N2 AS T2) -- 16 rows
	,N4(C) AS (SELECT 0 FROM N3 AS T1 CROSS JOIN N3 AS T2) -- 256 rows
	,N5(C) AS (SELECT 0 FROM N4 AS T1 CROSS JOIN N4 AS T2) -- 65,536 rows
	,N6(C) AS (SELECT 0 FROM N5 AS T1 CROSS JOIN N4 AS T2) -- 16,777,216 rows
	,IDs(ID) AS (SELECT ROW_NUMBER() OVER (ORDER BY (SELECT NULL)) FROM N6)
	,Info(OrderId, CustomerId, OrderDateOffset, RatePlanId, ServiceId, Pieces)
	as
	(
		select 
			ID, ID % @MaxCustomers + 1, ID % (365*24*60)
			,ID % 2 + 1, ID % 3 + 1, ID % 5 + 1
		from IDs 
		where ID <= @MaxOrderId
	)
	,Info2(OrderId, OrderDate, OrderNum, CustomerId, RatePlanId ,ServiceId
		,Pieces ,PickupAddressId, OrderStatusId)
	as
	(
		select 
			OrderId, DATEADD(MINUTE, -OrderDateOffset, GETDATE())
			,CONVERT(varchar(10),OrderId), CustomerId
			,RatePlanId, ServiceId, Pieces
			,(CustomerId - 1) * @MaxAddresses + OrderId % 20
			,CASE	
				WHEN OrderDateOffset > 5 * 24 * 60
				THEN 4
				ELSE OrderId % 4 + 1
			END
		from Info
	)	
	insert into Delivery.Orders(OrderDate, OrderNum, CustomerId,
		PickupAddressId, DeliveryAddressId, ServiceId, RatePlanId,
		OrderStatusId, DriverId, Pieces, Amount)
	select 
		o.OrderDate, o.OrderNum, o.CustomerId, o.PickupAddressId
		,CASE 
			WHEN o.PickupAddressId % @MaxAddresses = 0
			THEN o.PickupAddressId + 1
			ELSE o.PickupAddressId - 1
		END, o.ServiceId, o.RatePlanId, o.OrderStatusId
		,CASE
			WHEN o.OrderStatusId in (1,4)
			THEN NULL
			ELSE OrderId % @MaxDrivers + 1
		END, o.Pieces, r.Rate
	from Info2 o join Delivery.Rates r on o.ServiceId = r.ServiceId and o.RatePlanId = r.RatePlanId	
commit
go

-- CHECKS 
select count(*) from Delivery.Orders

select OrderStatusId, COUNT(*) from Delivery.Orders group by OrderStatusId

select * from Delivery.Orders o
where exists 
	(
		select * 
		from Delivery.Addresses a 
		where o.DeliveryAddressId = a.AddressId and o.CustomerId <> a.CustomerId
	) or
	exists 
	(
		select * 
		from Delivery.Addresses a 
		where o.PickupAddressId = a.AddressId and o.CustomerId <> a.CustomerId
	)	
go	
	