/****************************************************************************/
/*                         SQL Server Internals v3                          */
/*                           Training Materials                             */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                         Diagnostics Scripts                              */
/*                               CPU Load                                   */
/****************************************************************************/

declare
	 @now bigint;
	 
select @now = cpu_ticks / (cpu_ticks / ms_ticks) 
from sys.dm_os_sys_info with (nolock);

;with RingBufferData([timestamp], rec)
as
(
	select [timestamp], CONVERT(xml, record) as rec 
	from sys.dm_os_ring_buffers with (nolock)
	where 
		ring_buffer_type = N'RING_BUFFER_SCHEDULER_MONITOR' and
		record like N'%<SystemHealth>%'
)
,Data(id, SystemIdle, SQLCPU, [timestamp])
as
(
	select
		rec.value('(./Record/@id)[1]', 'int') 
		,rec.value('(./Record/SchedulerMonitorEvent/SystemHealth/SystemIdle)[1]', 'int')
		,rec.value('(./Record/SchedulerMonitorEvent/SystemHealth/ProcessUtilization)[1]', 'int') 
		,[timestamp] 
	from RingBufferData
)
select top 256 
	dateadd(ms, -1 * (@now - [timestamp]), getdate()) AS [Event Time] 
	,SQLCPU AS [SQL Server CPU Utilization] 
    ,SystemIdle AS [System Idle]
    ,100 - SystemIdle - SQLCPU AS [Other Processes CPU Utilization]
from Data
order by id desc
option (recompile, maxdop 1);
go

