/****************************************************************************/
/*                         SQL Server Internals v3                          */
/*                           Training Materials                             */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                        Locking and Blocking                              */
/*                        Deadlocks (Session 2)                             */
/****************************************************************************/

use SQLServerInternals
go

set nocount on
go

if exists
(
	select *
	from sys.procedures p join sys.schemas s on 
		p.schema_id = s.schema_id
	where
		s.name = 'dbo' and p.name = 'DoDeadlock'
)
	drop proc dbo.DoDeadlock
go

create proc dbo.DoDeadlock
as
begin
	set transaction isolation level read committed
	begin tran
		update dbo.SmallRow set CharField = 'b' where ID = 1000
		raiserror('First update is done',0,1) with nowait
		select * from dbo.SmallRow where IntField = 1001
	commit
end
go

exec dbo.DoDeadlock
go


declare
	@H varbinary(max) = 0x020000006fe9221996c1b4da6ebc13dd4f4ece40c370226f0000000000000000000000000000000000000000
	,@S int = 24
	,@E int = 122

SELECT TOP 50 
	SUBSTRING(qt.TEXT, (qs.statement_start_offset/2)+1,
		((
			CASE qs.statement_end_offset
				WHEN -1 THEN DATALENGTH(qt.TEXT)
				ELSE qs.statement_end_offset
			END - qs.statement_start_offset)/2)+1) as SQL,
	qs.execution_count,
	(qs.total_logical_reads + qs.total_logical_writes) / qs.execution_count as [Avg IO],
	qp.query_plan,
	qs.total_logical_reads, qs.last_logical_reads,
	qs.total_logical_writes, qs.last_logical_writes,
	qs.total_worker_time,
	qs.last_worker_time,
	qs.total_elapsed_time/1000 total_elapsed_time_in_ms,
	qs.last_elapsed_time/1000 last_elapsed_time_in_ms,
	qs.last_execution_time,
	qs.sql_handle,
	qs.statement_start_offset,
	qs.statement_end_offset
FROM 
	sys.dm_exec_query_stats qs with (nolock)
		CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) qt
		OUTER APPLY sys.dm_exec_query_plan(qs.plan_handle) qp
where
	qs.sql_handle = @H -- and qs.statement_start_offset = @S and qs.statement_end_offset = @E
option (recompile)  
