/****************************************************************************/
/*                         SQL Server Internals v3                          */
/*                           Training Materials                             */
/*                                                                          */
/*                  Written by Dmitri V. Korotkevitch                       */
/*                      http://aboutsqlserver.com                           */
/*                        dk@aboutsqlserver.com                             */
/****************************************************************************/
/*                        Locking and Blocking                              */
/*               Optimistic Isolation Levels and Fragmentation              */
/****************************************************************************/
use SQLServerInternals
go

set nocount on 
go

if exists(
	select *
	from sys.tables t join sys.schemas s on
		t.schema_id = s.schema_id
	where
		t.name = 'Positions' and s.name = 'dbo'
)
	drop table dbo.Positions
go

create table dbo.Positions
(
	ID int not null identity(1,1),
	Latitude decimal(9,6) not null,
	Longitude decimal(9,6) not null,
	Point geography null,
	Placeholder char(200)
		constraint DEF_Positions_Placeholder
		default 'Placeholder',
		
	constraint PK_Positions
	primary key clustered(ID)
)
go

;with Lats(Lat)
as
(
	select CONVERT(decimal(9,6),-90.0)
	
	union all
	
	select CONVERT(decimal(9,6),Lat + 1)
	from Lats
	where Lat < 90.0
)
,Lons(Lon)
as
(
	select CONVERT(decimal(9,6),-180.0)
	
	union all
	
	select CONVERT(decimal(9,6),Lon + 1)
	from Lons
	where Lon < 180.0
)
insert into dbo.Positions(Latitude, Longitude)
	select Lats.Lat, Lons.Lon
	from Lats cross join Lons
option (maxrecursion 0)
go



----- Snapshot isolation levels
create nonclustered index IDX_Positions_Latitude
on dbo.Positions(Latitude)
go

select page_count, avg_page_space_used_in_percent, avg_fragmentation_in_percent, 
	min_record_size_in_bytes, max_record_size_in_bytes, *
from sys.dm_db_index_physical_stats(DB_ID(),OBJECT_ID(N'Positions'),2,null,'DETAILED')
go

set transaction isolation level snapshot
begin tran
	update dbo.Positions set Latitude = 0
commit
go

select page_count, avg_page_space_used_in_percent, avg_fragmentation_in_percent, 
	min_record_size_in_bytes, max_record_size_in_bytes, *
from sys.dm_db_index_physical_stats(DB_ID(),OBJECT_ID(N'Positions'),2,null,'DETAILED')
go